@@ -7,36 +7,33 @@
    delete tagger; \
    return -1; }
 
-int main (int argc, char **argv) 
-{
-  char input[1024] = "ϺϼϺäƤܤֻҤϤ";
+// Sample of MeCab::Tagger class.
+int main (int argc, char **argv) {
+  char input[1024] = "太郎は次郎が持っている本を花子に渡した。";
 
-  MeCab::Tagger *tagger = MeCab::createTagger("-l1");
+  MeCab::Tagger *tagger = MeCab::createTagger("");
   CHECK(tagger);
 
+  // Gets tagged result in string format.
   const char *result = tagger->parse(input);
   CHECK(result);
   std::cout << "INPUT: " << input << std::endl;
   std::cout << "RESULT: " << result << std::endl;
 
+  // Gets N best results in string format.
   result = tagger->parseNBest(3, input);
-  CHECK(result); 
+  CHECK(result);
   std::cout << "NBEST: " << std::endl << result;
 
+  // Gets N best results in sequence.
   CHECK(tagger->parseNBestInit(input));
   for (int i = 0; i < 3; ++i) {
     std::cout << i << ":" << std::endl << tagger->next();
-  } 
-
-  const MeCab::Node* node = tagger->parseToNode(input);
-  CHECK(node);
-  for (; node; node = node->next) {
-    std::cout.write(node->surface, node->length);
   }
 
-  node = tagger->parseToNode(input);
+  // Gets Node object.
+  const MeCab::Node* node = tagger->parseToNode(input);
   CHECK(node);
-
   for (; node; node = node->next) {
     std::cout << node->id << ' ';
     if (node->stat == MECAB_BOS_NODE)
@@ -61,20 +58,7 @@ int main (int argc, char **argv)
 	      << ' ' << node->cost << std::endl;
   }
 
-  node = tagger->parseToNode(input);
-  size_t len = node->sentence_length;
-  for (int i = 0; i <= len; ++i) {
-    MeCab::Node *b = node->begin_node_list[i];
-    MeCab::Node *e = node->end_node_list[i];
-    for (; b; b = b->bnext) {
-      printf("B[%d] %s\t%s\n", i, b->surface, b->feature);
-    }
-    for (; e; e = e->enext) {
-      printf("E[%d] %s\t%s\n", i, e->surface, e->feature);
-    }
-  }
-
-
+  // Dictionary info.
   const MeCab::DictionaryInfo *d = tagger->dictionary_info();
   for (; d; d = d->next) {
     std::cout << "filename: " <<  d->filename << std::endl;
@@ -86,8 +70,7 @@ int main (int argc, char **argv)
     std::cout << "version: " <<  d->version << std::endl;
   }
 
-
   delete tagger;
-   
+
   return 0;
 }
