@@ -1,5 +1,4 @@
 //  MeCab -- Yet Another Part-of-Speech and Morphological Analyzer
-//  $Id: dictionary_compiler.cpp 173 2009-04-18 08:10:57Z taku-ku $;
 //
 //  Copyright(C) 2001-2006 Taku Kudo <taku@chasen.org>
 //  Copyright(C) 2004-2006 Nippon Telegraph and Telephone Corporation
@@ -7,12 +6,12 @@
 #include <map>
 #include <vector>
 #include <string>
-#include "mecab.h"
-#include "dictionary_rewriter.h"
 #include "char_property.h"
-#include "param.h"
 #include "connector.h"
 #include "dictionary.h"
+#include "dictionary_rewriter.h"
+#include "mecab.h"
+#include "param.h"
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -80,7 +79,7 @@ class DictionaryComplier {
     if (!userdic.empty()) {
       CHECK_DIE(dic.size()) << "no dictionaries are specified";
 
-      param.set("type", MECAB_USR_DIC);
+      param.set("type", static_cast<int>(MECAB_USR_DIC));
       Dictionary::compile(param, dic,
                           DCONF(MATRIX_DEF_FILE),
                           DCONF(MATRIX_FILE),
@@ -103,7 +102,7 @@ class DictionaryComplier {
       if (opt_unknown) {
         std::vector<std::string> tmp;
         tmp.push_back(DCONF(UNK_DEF_FILE));
-        param.set("type", MECAB_UNK_DIC);
+        param.set("type", static_cast<int>(MECAB_UNK_DIC));
         Dictionary::compile(param, tmp,
                             DCONF(MATRIX_DEF_FILE),
                             DCONF(MATRIX_FILE),
@@ -116,7 +115,7 @@ class DictionaryComplier {
 
       if (opt_sysdic) {
         CHECK_DIE(dic.size()) << "no dictionaries are specified";
-        param.set("type", MECAB_SYS_DIC);
+        param.set("type", static_cast<int>(MECAB_SYS_DIC));
         Dictionary::compile(param, dic,
                             DCONF(MATRIX_DEF_FILE),
                             DCONF(MATRIX_FILE),
