@@ -1,6 +1,5 @@
 // MeCab -- Yet Another Part-of-Speech and Morphological Analyzer
 //
-//  $Id: mmap.h 173 2009-04-18 08:10:57Z taku-ku $;
 //
 //  Copyright(C) 2001-2006 Taku Kudo <taku@chasen.org>
 //  Copyright(C) 2004-2006 Nippon Telegraph and Telephone Corporation
@@ -49,6 +48,7 @@ extern "C" {
 }
 
 #include "common.h"
+#include "utils.h"
 
 #ifndef O_BINARY
 #define O_BINARY 0
@@ -108,33 +108,33 @@ template <class T> class Mmap {
       mode2 = PAGE_READWRITE;
       mode3 = FILE_MAP_ALL_ACCESS;
     } else {
-      CHECK_CLOSE_FALSE(false) << "unknown open mode:" << filename;
+      CHECK_FALSE(false) << "unknown open mode:" << filename;
     }
 
-    hFile = CreateFile(filename, mode1, FILE_SHARE_READ, 0,
-                       OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
-    CHECK_CLOSE_FALSE(hFile != INVALID_HANDLE_VALUE)
+    hFile = ::CreateFileW(WPATH(filename), mode1, FILE_SHARE_READ, 0,
+                          OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
+    CHECK_FALSE(hFile != INVALID_HANDLE_VALUE)
         << "CreateFile() failed: " << filename;
 
-    length = GetFileSize(hFile, 0);
+    length = ::GetFileSize(hFile, 0);
 
-    hMap = CreateFileMapping(hFile, 0, mode2, 0, 0, 0);
-    CHECK_CLOSE_FALSE(hMap) << "CreateFileMapping() failed: " << filename;
+    hMap = ::CreateFileMapping(hFile, 0, mode2, 0, 0, 0);
+    CHECK_FALSE(hMap) << "CreateFileMapping() failed: " << filename;
 
-    text = reinterpret_cast<T *>(MapViewOfFile(hMap, mode3, 0, 0, 0));
-    CHECK_CLOSE_FALSE(text) << "MapViewOfFile() failed: " << filename;
+    text = reinterpret_cast<T *>(::MapViewOfFile(hMap, mode3, 0, 0, 0));
+    CHECK_FALSE(text) << "MapViewOfFile() failed: " << filename;
 
     return true;
   }
 
   void close() {
-    if (text) { UnmapViewOfFile(text); }
+    if (text) { ::UnmapViewOfFile(text); }
     if (hFile != INVALID_HANDLE_VALUE) {
-      CloseHandle(hFile);
+      ::CloseHandle(hFile);
       hFile = INVALID_HANDLE_VALUE;
     }
     if (hMap) {
-      CloseHandle(hMap);
+      ::CloseHandle(hMap);
       hMap = 0;
     }
     text = 0;
@@ -154,12 +154,12 @@ template <class T> class Mmap {
     else if (std::strcmp(mode, "r+") == 0)
       flag = O_RDWR;
     else
-      CHECK_CLOSE_FALSE(false) << "unknown open mode: " << filename;
+      CHECK_FALSE(false) << "unknown open mode: " << filename;
 
-    CHECK_CLOSE_FALSE((fd = open__(filename, flag | O_BINARY)) >= 0)
+    CHECK_FALSE((fd = open__(filename, flag | O_BINARY)) >= 0)
         << "open failed: " << filename;
 
-    CHECK_CLOSE_FALSE(fstat(fd, &st) >= 0)
+    CHECK_FALSE(fstat(fd, &st) >= 0)
         << "failed to get file size: " << filename;
 
     length = st.st_size;
@@ -168,7 +168,7 @@ template <class T> class Mmap {
     int prot = PROT_READ;
     if (flag == O_RDWR) prot |= PROT_WRITE;
     char *p;
-    CHECK_CLOSE_FALSE((p = reinterpret_cast<char *>
+    CHECK_FALSE((p = reinterpret_cast<char *>
                        (mmap(0, length, prot, MAP_SHARED, fd, 0)))
                       != MAP_FAILED)
         << "mmap() failed: " << filename;
@@ -176,7 +176,7 @@ template <class T> class Mmap {
     text = reinterpret_cast<T *>(p);
 #else
     text = new T[length];
-    CHECK_CLOSE_FALSE(read(fd, text, length) >= 0)
+    CHECK_FALSE(read(fd, text, length) >= 0)
         << "read() failed: " << filename;
 #endif
     close__(fd);
