@@ -1,28 +1,24 @@
 //   MeCab -- Yet Another Part-of-Speech and Morphological Analyzer
 //
-//   $Id: utils.h 173 2009-04-18 08:10:57Z taku-ku $;
 //
 //   Copyright(C) 2001-2006 Taku Kudo <taku@chasen.org>
 //   Copyright(C) 2004-2006 Nippon Telegraph and Telephone Corporation
 #ifndef MECAB_UTILS_H
 #define MECAB_UTILS_H
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
+#include <algorithm>
 #include <cstdlib>
 #include <cstdio>
 #include <cstring>
-#include <algorithm>
 #include <string>
 #include <vector>
 #include "common.h"
 
-namespace MeCab {
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
-template <class T> inline T _min(T x, T y) { return(x < y) ? x : y; }
-template <class T> inline T _max(T x, T y) { return(x > y) ? x : y; }
+namespace MeCab {
 
 class Param;
 
@@ -90,6 +86,8 @@ inline void read_static(const char **ptr, T& value) {
   memcpy(&value, r, sizeof(T));
 }
 
+int load_request_type(const Param &param);
+
 bool load_dictionary_resource(Param *);
 
 bool escape_csv_element(std::string *w);
@@ -119,8 +117,7 @@ inline size_t tokenizeCSV(char *str,
 
   for (; str < eos; ++str) {
     // skip white spaces
-    while (*str == ' ' || *str == '\t') ++str;
-    bool inquote = false;
+     while (*str == ' ' || *str == '\t') ++str;
     if (*str == '"') {
       start = ++str;
       end = start;
@@ -132,7 +129,6 @@ inline size_t tokenizeCSV(char *str,
         }
         *end++ = *str;
       }
-      inquote = true;
       str = std::find(str, eos, ',');
     } else {
       start = str;
@@ -193,8 +189,8 @@ inline double logsumexp(double x, double y, bool flg) {
 #define MINUS_LOG_EPSILON  50
 
   if (flg) return y;  // init mode
-  double vmin = _min(x, y);
-  double vmax = _max(x, y);
+  double vmin = std::min<double>(x, y);
+  double vmax = std::max<double>(x, y);
   if (vmax > vmin + MINUS_LOG_EPSILON) {
     return vmax;
   } else {
@@ -219,5 +215,10 @@ inline char getEscapedChar(const char p) {
 
   return '\0';  // never be here
 }
+
+#if defined(_WIN32) && !defined(__CYGWIN__)
+std::wstring Utf8ToWide(const std::string &input);
+std::string WideToUtf8(const std::wstring &input);
+#endif
 }
 #endif
